@@ -30,11 +30,11 @@ class BaseAPIParser:
         _model_folder = os.path.join("%s", "%s", "saved_models")
         _csv_folder = os.path.join("%s", "%s", "saved_csvs")
         _tensorboard_folder = os.path.join("%s", "%s", "tensorboard_logs")
-        self.sub_experiment_dirs = [
-            _model_folder,
-            _csv_folder,
-            _tensorboard_folder
-        ]
+        self.sub_experiment_dirs = {
+            "models": _model_folder,
+            "csvs": _csv_folder,
+            "tensorboard": _tensorboard_folder
+        }
 
         self.trainer, self.tester_obj = None, None
 
@@ -46,7 +46,17 @@ class BaseAPIParser:
         self.set_num_epochs_dict()
         self.make_sub_experiment_dirs()
         self.set_meta_record_keeper()
-        if self.args.evaluate and self.args.meta_testing_method:
+        if self.args.evaluate:
+            return_dict = {}
+            for self.curr_meta_testing_method in c_f.if_str_convert_to_singleton_list(self.args.meta_testing_method):
+                return_dict[self.curr_meta_testing_method] = self.run_train_or_eval()
+            return return_dict
+        else:
+            return self.run_train_or_eval()
+
+
+    def run_train_or_eval(self):
+        if self.args.evaluate and self.get_curr_meta_testing_method():
             self.meta_eval()
         else:
             try:
@@ -109,17 +119,18 @@ class BaseAPIParser:
         c_f.makedir_if_not_there(self.experiment_folder)
 
     def make_sub_experiment_dirs(self):
-        for s in self.sub_experiment_dirs:
+        for s in self.sub_experiment_dirs.values():
             for r in self.split_manager.split_scheme_names:
                 c_f.makedir_if_not_there(s % (self.experiment_folder, r))
 
     def set_curr_folders(self):
-        self.model_folder, self.csv_folder, self.tensorboard_folder = self.get_sub_experiment_dir_paths()[self.split_manager.curr_split_scheme_name]
+        folders = self.get_sub_experiment_dir_paths()[self.split_manager.curr_split_scheme_name]
+        self.model_folder, self.csv_folder, self.tensorboard_folder = folders["models"], folders["csvs"], folders["tensorboard"]
 
     def get_sub_experiment_dir_paths(self):
         sub_experiment_dir_paths = {}
         for k in self.split_manager.split_scheme_names:
-            sub_experiment_dir_paths[k] = [s % (self.experiment_folder, k) for s in self.sub_experiment_dirs]
+            sub_experiment_dir_paths[k] = {folder_type: s % (self.experiment_folder, k) for folder_type, s in self.sub_experiment_dirs.items()}
         return sub_experiment_dir_paths
 
     def set_num_epochs_dict(self):
@@ -132,7 +143,7 @@ class BaseAPIParser:
         self.latest_sub_experiment_epochs = c_f.latest_sub_experiment_epochs(self.get_sub_experiment_dir_paths())
         latest_epochs = list(self.latest_sub_experiment_epochs.values())
         if self.is_training():
-            c_f.save_config_files(self.args.place_to_save_configs, self.args.dict_of_yamls, self.args.resume_training, self.args.reproduce_results, latest_epochs)
+            c_f.save_config_files(self.args.place_to_save_configs, self.args.dict_of_yamls, self.args.resume_training, latest_epochs)
         delattr(self.args, "dict_of_yamls")
         delattr(self.args, "place_to_save_configs")
 
@@ -317,7 +328,8 @@ class BaseAPIParser:
     def set_meta_record_keeper(self):
         is_new_experiment = self.beginning_of_training()
         if len(self.split_manager.split_scheme_names) > 1:
-            _, csv_folder, tensorboard_folder = [s % (self.experiment_folder, "meta_logs") for s in self.sub_experiment_dirs]
+            folders = {folder_type: s % (self.experiment_folder, "meta_logs") for folder_type, s in self.sub_experiment_dirs.items()}
+            csv_folder, tensorboard_folder = folders["csvs"], folders["tensorboard"]
             self.meta_record_keeper, _, _ = logging_presets.get_record_keeper(csv_folder, tensorboard_folder,  self.global_db_path, self.args.experiment_name, is_new_experiment)
             self.meta_accuracies = defaultdict(lambda: defaultdict(lambda: defaultdict(dict)))
 
@@ -335,7 +347,7 @@ class BaseAPIParser:
     def record_meta_logs(self):
         if hasattr(self, "meta_accuracies") and len(self.meta_accuracies) > 0:
             for split in self.args.splits_to_eval:
-                group_name = self.get_eval_record_name_dict("meta")[split]
+                group_name = self.get_eval_record_name_dict("meta_SeparateEmbeddings")[split]
                 len_of_existing_records = c_f.try_getting_db_count(self.meta_record_keeper, group_name)
                 for i, is_trained in enumerate([0, 1]):
                     curr_dict = self.meta_accuracies[split][is_trained]
@@ -351,7 +363,7 @@ class BaseAPIParser:
 
     def return_val_accuracy_and_standard_error(self):
         if hasattr(self, "meta_record_keeper"):
-            group_name = self.get_eval_record_name_dict("meta")["val"]
+            group_name = self.get_eval_record_name_dict("meta_SeparateEmbeddings")["val"]
             def get_average_best_and_sem(key):
                 is_trained = 1
                 sem_key = "SEM_%s"%key
@@ -499,8 +511,7 @@ class BaseAPIParser:
         return trunk, embedder
 
     def meta_eval(self):
-        assert self.args.splits_to_eval == ["test"]
-        meta_model_getter = getattr(self, "meta_"+self.args.meta_testing_method)
+        meta_model_getter = getattr(self, self.get_curr_meta_testing_method())
         self.models = {}
         self.record_keeper = self.meta_record_keeper
         self.hooks = logging_presets.HookContainer(self.record_keeper, record_group_name_prefix=meta_model_getter.__name__, primary_metric=self.args.eval_primary_metric)
@@ -524,27 +535,37 @@ class BaseAPIParser:
                 self.record_keeper.update_records({"is_trained": int(i==1)}, global_iteration=global_iteration, input_group_name_for_non_objects=group_name)
                 self.record_keeper.update_records({"timestamp": c_f.get_datetime()}, global_iteration=global_iteration, input_group_name_for_non_objects=group_name)
 
-                for irrelevant_key in ["epoch", "best_epoch", "best_accuracy"]:
-                    self.record_keeper.record_writer.records.pop(irrelevant_key, None)
+                for irrelevant_key in ["best_epoch", "best_accuracy"]:
+                    self.record_keeper.record_writer.records[group_name].pop(irrelevant_key, None)
                 self.record_keeper.save_records()
 
 
     def get_eval_record_name_dict(self, eval_type="non_meta", return_all=False):
+        if not getattr(self, "hooks", None):
+            self.hooks = logging_presets.HookContainer(None, primary_metric=self.args.eval_primary_metric)
+        if not getattr(self, "tester_obj", None):
+            self.tester_obj = self.pytorch_getter.get("tester", self.args.testing_method, self.get_tester_kwargs())
         prefix = self.hooks.record_group_name_prefix 
         self.hooks.record_group_name_prefix = "" #temporary
         non_meta = {k:self.hooks.record_group_name(self.tester_obj, k) for k in ["train", "val", "test"]}
-        meta = {k:"meta_"+v for k,v in non_meta.items()}
-        meta_concatenated = {k:"meta_ConcatenateEmbeddings_"+v for k,v in non_meta.items()}
+        meta_separate = {k:"meta_SeparateEmbeddings_"+v for k,v in non_meta.items()}
+        meta_concatenate = {k:"meta_ConcatenateEmbeddings_"+v for k,v in non_meta.items()}
         self.hooks.record_group_name_prefix = prefix
 
         name_dict = {"non_meta": non_meta,
-                    "meta": meta,
-                    "meta_ConcatenateEmbeddings": meta_concatenated}
+                    "meta_SeparateEmbeddings": meta_separate,
+	                "meta_ConcatenateEmbeddings": meta_concatenate}
 
         if return_all:
             return name_dict
         return name_dict[eval_type]
 
 
+    def get_curr_meta_testing_method(self):
+        # SeparateEmbeddings is equivalent to the regular per-split eval
+        if self.curr_meta_testing_method == "meta_SeparateEmbeddings":
+            return None
+        return self.curr_meta_testing_method
+
 
 
