@@ -7,6 +7,9 @@ from ax.utils.report.render import render_report_elements
 from ax.plot.contour import interact_contour
 from ax.modelbridge.registry import Models
 from ax.core.base_trial import TrialStatus
+from ax.plot.slice import interact_slice
+from ax.plot.helper import get_range_parameters
+from ax.core.parameter import RangeParameter, ParameterType
 import re
 from easy_module_attribute_getter import utils as emag_utils, YamlReader
 import glob
@@ -14,6 +17,7 @@ import os
 import csv
 import pandas as pd
 from ..utils import common_functions as c_f
+from .base_runner import BaseRunner
 from .single_experiment_runner import SingleExperimentRunner
 import pytorch_metric_learning.utils.logging_presets as logging_presets
 import numpy as np
@@ -24,6 +28,7 @@ import json
 logging.info("Done importing packages in bayes_opt_runner")
 
 BAYESIAN_KEYWORDS=("~BAYESIAN~", "~LOG_BAYESIAN~", "~INT_BAYESIAN~")
+RESUME_FAILURE="RESUME_FAILURE"
 
 
 def set_optimizable_params_and_bounds(args_dict, bayes_params, parent_key, keywords=BAYESIAN_KEYWORDS):
@@ -70,9 +75,10 @@ def remove_keywords(YR):
         emag_utils.remove_key_word_recursively(YR.args.__dict__, keyword)
 
 
-class BayesOptRunner(SingleExperimentRunner):
+class BayesOptRunner(BaseRunner):
     def __init__(self, bayes_opt_iters, reproductions, **kwargs):
         super().__init__(**kwargs)
+        self.set_YR(use_super=True)
         self.bayes_opt_iters = bayes_opt_iters
         self.reproductions = reproductions
         self.experiment_name = self.YR.args.experiment_name
@@ -87,7 +93,6 @@ class BayesOptRunner(SingleExperimentRunner):
         self.accuracy_report_detailed_filename = os.path.join(self.bayes_opt_root_experiment_folder, "accuracy_report_detailed.yaml")
         self.accuracy_report_filename = os.path.join(self.bayes_opt_root_experiment_folder, "accuracy_report.yaml")
         self.bayes_opt_table_name = "bayes_opt"
-        self.original_config_folder = "%s/original_configs"%(self.bayes_opt_root_experiment_folder)
         self.set_YR(use_super=False)
 
     def set_YR(self, use_super=True):
@@ -100,11 +105,13 @@ class BayesOptRunner(SingleExperimentRunner):
         ax_client = self.get_ax_client()
         trials = ax_client.experiment.trials
         record_keeper, _, _ = logging_presets.get_record_keeper(self.csv_folder, self.tensorboard_folder)
+        temp_YR_for_config_diffs = self.read_yaml_and_find_bayes(find_bayes_params=False)
 
         for i in range(0, self.bayes_opt_iters):
             if i in trials and trials[i].status == TrialStatus.COMPLETED:
                 continue
             logging.info("Optimization iteration %d"%i)
+            c_f.save_config_files(self.YR.args.place_to_save_configs, temp_YR_for_config_diffs.args.dict_of_yamls, True, [i]) # save config diffs, if any
             sub_experiment_name = self.get_sub_experiment_name(i)
             parameters, trial_index, experiment_func = self.get_parameters_and_trial_index(ax_client, sub_experiment_name, i)
             ax_client.complete_trial(trial_index=trial_index, raw_data=experiment_func(parameters, sub_experiment_name))
@@ -182,16 +189,18 @@ class BayesOptRunner(SingleExperimentRunner):
         return best_sub_experiment_name
 
     def create_accuracy_report(self, best_sub_experiment_name):
+        dummy_api_parser = self.get_api_parser(self.YR.args)
+        eval_record_group_dicts = dummy_api_parser.get_eval_record_name_dict(return_all=True)
         global_record_keeper, _, _ = logging_presets.get_record_keeper(self.csv_folder, self.tensorboard_folder, self.global_db_path, "", False)
         exp_names = glob.glob(os.path.join(self.bayes_opt_root_experiment_folder, "%s*"%best_sub_experiment_name))
 
         exp_names = [os.path.basename(e) for e in exp_names]
         results, summary = {}, {}
 
-        for eval_type in ["meta", "meta_ConcatenateEmbeddings"]:
+        for eval_type in c_f.if_str_convert_to_singleton_list(self.YR.args.meta_testing_method):
             results[eval_type] = {}
             summary[eval_type] = collections.defaultdict(lambda: collections.defaultdict(list))
-            table_name = self.eval_record_group_dicts[eval_type]["test"]
+            table_name = eval_record_group_dicts[eval_type]["test"]
             
             for exp in exp_names:
                 results[eval_type][exp] = {}
@@ -210,7 +219,7 @@ class BayesOptRunner(SingleExperimentRunner):
                     q_as_dict = dict(v1)
                     results[eval_type][exp][is_trained] = q_as_dict
                     for acc_key, v2 in q_as_dict.items():
-                        if all(not acc_key.startswith(x) for x in ["is_trained", "best_epoch", "best_accuracy", "SEM", "id", "experiment_id", "timestamp"]):
+                        if all(not acc_key.startswith(x) for x in ["epoch", "is_trained", "SEM", "id", "experiment_id", "timestamp"]):
                             summary[eval_type][is_trained][acc_key].append(v2)
 
 
@@ -228,13 +237,19 @@ class BayesOptRunner(SingleExperimentRunner):
         c_f.write_yaml(self.accuracy_report_filename, json.loads(json.dumps(summary)), open_as="w")
 
 
-
+    def update_bayes_opt_search_space(self, ax_client):
+        for bp in self.bayes_params:
+            kwargs_dict = {"name": bp["name"], "lower": bp["bounds"][0], "upper": bp["bounds"][1]}
+            kwargs_dict["parameter_type"] = ParameterType.FLOAT if bp["value_type"] == "float" else ParameterType.INT
+            kwargs_dict["log_scale"] = bp["log_scale"]
+            ax_client.experiment.search_space.update_parameter(RangeParameter(**kwargs_dict))
 
     def get_ax_client(self):
         log_paths = self.get_all_log_paths()
         ax_client = None
         if len(log_paths) > 0:
             ax_client = open_log(log_paths)
+            self.update_bayes_opt_search_space(ax_client)
         if ax_client is None:
             ax_client = AxClient()
             ax_client.create_experiment(parameters=self.bayes_params, name=self.YR.args.experiment_name, minimize=False, objective_name=self.YR.args.eval_primary_metric)
@@ -243,27 +258,34 @@ class BayesOptRunner(SingleExperimentRunner):
 
     def plot_progress(self, ax_client):
         model = Models.GPEI(experiment=ax_client.experiment, data=ax_client.experiment.fetch_data())
-        html_elements = []
-        html_elements.append(plot_config_to_html(ax_client.get_optimization_trace()))
-        try:
+        html_elements = [plot_config_to_html(ax_client.get_optimization_trace())]
+        model_params = get_range_parameters(model)
+        if len(model_params) > 1:
             html_elements.append(plot_config_to_html(interact_contour(model=model, metric_name=self.YR.args.eval_primary_metric)))
-        except IndexError:
-            logging.warning("Can't create contour plot with only 1 parameter")
+        else:
+            html_elements.append(plot_config_to_html(interact_slice(model=model, param_name=model_params[0].name, metric_name=self.YR.args.eval_primary_metric)))
         with open(os.path.join(self.bayes_opt_root_experiment_folder, "optimization_plots.html"), 'w') as f:
             f.write(render_report_elements(self.experiment_name, html_elements))
 
 
-    def read_yaml_and_find_bayes(self):
+    def read_yaml_and_find_bayes(self, find_bayes_params=True):
         YR = self.setup_yaml_reader()
-        config_paths = self.get_saved_config_paths(self.original_config_folder) if os.path.isdir(self.original_config_folder) else self.get_root_config_paths(YR.args)
-        if not os.path.isdir(self.original_config_folder):
-            YR.args, _, YR.args.dict_of_yamls = YR.load_yamls(config_paths=config_paths, max_merge_depth=float('inf'), merge_argparse=True)
-            c_f.save_config_files(self.original_config_folder, YR.args.dict_of_yamls, False, False, [])
-        else:
-            YR.args, _, YR.args.dict_of_yamls = YR.load_yamls(config_paths=config_paths, max_merge_depth=float('inf'), merge_argparse=False)
-        bayes_params = []
-        set_optimizable_params_and_bounds(YR.args.__dict__, bayes_params, '')
-        return YR, bayes_params
+        bayes_opt_config_exists = os.path.isdir(YR.args.place_to_save_configs)
+
+        config_paths = self.get_saved_config_paths(YR.args) if bayes_opt_config_exists else self.get_root_config_paths(YR.args)
+        merge_argparse = self.merge_argparse_when_resuming if bayes_opt_config_exists else True
+        YR.args, _, YR.args.dict_of_yamls = YR.load_yamls(config_paths = config_paths, 
+                                                        max_merge_depth = float('inf'), 
+                                                        merge_argparse = merge_argparse)
+
+        if not bayes_opt_config_exists:                                         
+            c_f.save_config_files(YR.args.place_to_save_configs, YR.args.dict_of_yamls, False, [])
+
+        if find_bayes_params:
+            bayes_params = []
+            set_optimizable_params_and_bounds(YR.args.__dict__, bayes_params, '')
+            return YR, bayes_params
+        return YR
 
 
     def set_experiment_name_and_place_to_save_configs(self, YR):
@@ -271,14 +293,18 @@ class BayesOptRunner(SingleExperimentRunner):
         YR.args.place_to_save_configs = os.path.join(YR.args.experiment_folder, "configs")
 
 
-    def get_simplified_yaml_reader(self, experiment_name):
-        YR = YamlReader()
-        YR.args, _ = self.setup_argparser().parse_known_args() # we want to ignore the "unknown" args in this case
-        YR.args.dataset_root = self.dataset_root
-        YR.args.experiment_name = experiment_name
-        self.set_experiment_name_and_place_to_save_configs(YR)
-        return YR
+    def starting_fresh(self, experiment_name):
+        def _starting_fresh(YR):
+            emag_utils.remove_dicts(YR.args.__dict__)
+            YR.args.dataset_root = self.dataset_root
+            YR.args.experiment_name = experiment_name
+            self.set_experiment_name_and_place_to_save_configs(YR)
+        return _starting_fresh
 
+    def get_simplified_yaml_reader(self, experiment_name):
+        YR = self.setup_yaml_reader()
+        self.starting_fresh(experiment_name)(YR)
+        return YR
 
     def delete_sub_experiment_folder(self, sub_experiment_name):
         logging.warning("Deleting and starting fresh for %s"%sub_experiment_name)
@@ -287,15 +313,17 @@ class BayesOptRunner(SingleExperimentRunner):
         global_record_keeper.record_writer.global_db.delete_experiment(sub_experiment_name)
 
 
-    def try_resuming(self, YR):
+    def try_resuming(self, YR, reproduction=False):
         try:
-            output = super().run_new_experiment(YR)
+            SER = self.get_single_experiment_runner()
+            starting_fresh_hook = self.starting_fresh(YR.args.experiment_name)
+            output = SER.reproduce_results(YR, starting_fresh_hook=starting_fresh_hook) if reproduction else SER.run_new_experiment_or_resume(YR)
         except Exception as e:
             YR.args.resume_training = None
             logging.error(repr(e))
             logging.warning("Could not resume training for %s"%YR.args.experiment_name)
             self.delete_sub_experiment_folder(YR.args.experiment_name)
-            output = None
+            output = RESUME_FAILURE
         return output
 
 
@@ -313,8 +341,8 @@ class BayesOptRunner(SingleExperimentRunner):
             self.delete_sub_experiment_folder(sub_experiment_name)
             parameter_load_successful = False
 
-        output = self.try_resuming(local_YR) if parameter_load_successful else None
-        return output if output is not None else self.run_new_experiment(parameters, sub_experiment_name)
+        output = self.try_resuming(local_YR) if parameter_load_successful else RESUME_FAILURE
+        return self.run_new_experiment(parameters, sub_experiment_name) if output == RESUME_FAILURE else output
 
 
     def run_new_experiment(self, parameters, sub_experiment_name):
@@ -328,17 +356,17 @@ class BayesOptRunner(SingleExperimentRunner):
         self.set_experiment_name_and_place_to_save_configs(local_YR)
         c_f.makedir_if_not_there(local_YR.args.experiment_folder)
         c_f.write_yaml(self.get_sub_experiment_bayes_opt_filename(local_YR.args.experiment_folder), parameters, open_as='w')
-        return self.start_experiment(local_YR.args)
+        SER = self.get_single_experiment_runner()
+        return SER.start_experiment(local_YR.args)
 
 
     def test_model(self, sub_experiment_name):
-        for meta_testing_method in [None, "ConcatenateEmbeddings"]:
-            local_YR = self.get_simplified_yaml_reader(sub_experiment_name)
-            local_YR.args.evaluate = True
-            local_YR.args.resume_training = None
-            local_YR.args.splits_to_eval = ["test"]
-            local_YR.args.__dict__["meta_testing_method~OVERRIDE~"] = meta_testing_method
-            super().run_new_experiment(local_YR)
+        local_YR = self.get_simplified_yaml_reader(sub_experiment_name)
+        local_YR.args.evaluate = True
+        local_YR.args.resume_training = None
+        local_YR.args.splits_to_eval = ["test"]
+        SER = self.get_single_experiment_runner()
+        SER.run_new_experiment_or_resume(local_YR)
 
 
     def reproduce_results(self, sub_experiment_name):
@@ -350,14 +378,25 @@ class BayesOptRunner(SingleExperimentRunner):
             local_YR = self.get_simplified_yaml_reader("%s_reproduction%d"%(sub_experiment_name, i))
             local_YR.args.reproduce_results = self.get_sub_experiment_path(sub_experiment_name)
             local_YR.args.resume_training = None
-            output = None
+            output = RESUME_FAILURE
             if os.path.isdir(local_YR.args.experiment_folder):
                 local_YR.args.resume_training = self.get_resume_training_value()
-                output = self.try_resuming(local_YR)
-            if output is None:
-                super().reproduce_results(local_YR)
+                output = self.try_resuming(local_YR, reproduction=True)
+            if output == RESUME_FAILURE:
+                SER = self.get_single_experiment_runner()
+                starting_fresh_hook = self.starting_fresh(local_YR.args.experiment_name)
+                SER.reproduce_results(local_YR, starting_fresh_hook=starting_fresh_hook)
             self.test_model(local_YR.args.experiment_name)
 
 
     def get_resume_training_value(self):
-        return "latest" if self.YR.args.resume_training is None else self.YR.args.resume_training
\ No newline at end of file
+        return "latest" if self.YR.args.resume_training is None else self.YR.args.resume_training
+
+    
+    def get_single_experiment_runner(self):
+        return SingleExperimentRunner(root_experiment_folder=self.bayes_opt_root_experiment_folder, 
+                                    root_config_folder=self.YR.args.place_to_save_configs, 
+                                    dataset_root=self.dataset_root,
+                                    pytorch_home=self.pytorch_home, 
+                                    global_db_path=self.global_db_path,
+                                    merge_argparse_when_resuming=self.merge_argparse_when_resuming)
\ No newline at end of file
